% -- BPPM_TH_mod.m
function [PPM_TH_seq, TH_seq] = BPPM_TH_mod(a, fs, Tc, Tf, epsilon, c)
% INPUT: sampling freq, chip time, frame size, PPM dither, TH code
% OUTPUT: modulated signal PPM and TH only

% Cyclic repetition of c: [c[0] c[1] ... c[M] c[0] c[1] ... c[M] c[0] ...] 
c_vec = reshape(c.' * ones(1,ceil(length(a)/length(c))), 1, []);
c_vec = c_vec(1:length(a));

% Dithering [s]
eta = c_vec * Tc;
theta = eta + a*epsilon;

% Dithering [idx]
dt = 1/fs;
eta_idx=ceil(eta/dt);
theta_idx = ceil(theta/dt); % # samples for the shifts

Tf_idx = ceil(Tf/dt); % # samples for each time frame
signal_idx = length(a)*Tf_idx; % # samples representing the signal
TH_seq = zeros(1,signal_idx);
PPM_TH_seq = zeros(1,signal_idx);

pos_TH = (1:Tf_idx:signal_idx)+eta_idx;
pos_THPPM = (1:Tf_idx:signal_idx) + theta_idx;
PPM_TH_seq(pos_THPPM) = ones(1,length(pos_THPPM));

TH_seq(pos_TH) = ones(1,length(pos_TH));