%% Main UWB receivers in AWGN - Soft vs. Hard decoding

clear all
clc
close all;
constant_ExN0=true;
powdBm = -30;
smp_freq = 50e9;
nBits = 5e4;
Ts = 3e-9;
Ns = [1 3];
Tc = 1e-9;
Nh = 3;
Np = 60000;
IR_d = 0.5e-9;
tau = 0.25e-9;
dPPM = 0.5e-9;
if(constant_ExN0)
    ExN0_dB = 0:2:10;
else
    EbN0_dB=0:2:10;
end
mod = 1; %mod==1 for PPM, mod == 2 for PAM

%% PPM Signal 1 - No repetition Ns=1
if(~constant_ExN0)
    ExN0_dB=EbN0_dB;
end
ExN0=10.^(ExN0_dB/10);
BER_theory_Ns_1=0.5*erfc(sqrt(0.5*ExN0));

[data1, c1, signal1, TH_signal1] = TX_BPPM_TH(nBits, Ns(1), ...
    smp_freq, Tc, Ts, Np, Nh, dPPM, IR_d, tau, powdBm);
[sRX1, a1] = pathloss(signal1, 10^(-30/20), 10, 2);
num_bits1 = length(data1);
mask1 = corrmask(TH_signal1,smp_freq, num_bits1*Ns(1), dPPM);
[sRXn1, noise1] = gnoise(sRX1, ExN0_dB, num_bits1*Ns(1));
[detected_bits1, BER1] = mod_receiver(sRXn1,mask1,smp_freq,data1,Ns(1),Ts,mod,2);


%% %% PPM Signal 2 - # repetitions = 3 - Hard Decision
if(~constant_ExN0)
    ExN0_dB=EbN0_dB-10*log10(Ns(2));
end
ExN0=10.^(ExN0_dB/10);
BER_theory_hard_Ns_3=zeros(1,length(ExN0_dB));
Pr_0=0.5*erfc(sqrt(0.5*ExN0));
for q=ceil(Ns(2)/2):Ns(2)
    BER_theory_hard_Ns_3=BER_theory_hard_Ns_3 + nchoosek(Ns(2),q)*((Pr_0).^q).*((1-Pr_0).^(Ns(2)-q));
end
[data2, c2, signal2, TH_signal2] = TX_BPPM_TH(nBits, Ns(2), ...
    smp_freq, Tc, Ts, Np, Nh, dPPM, IR_d, tau, powdBm);
[sRX2, a2] = pathloss(signal2, 10^(-30/20), 10, 2);
num_bits2 = length(data2);
mask2 = corrmask(TH_signal2,smp_freq, num_bits2*Ns(2), dPPM);
[sRXn2, noise2] = gnoise(sRX2, ExN0_dB, num_bits2*Ns(2));
[detected_bits2, BER2] = mod_receiver(sRXn2,mask2,smp_freq,data2,Ns(2),Ts,mod,1);


%% PPM Signal 3 - # repetitions = 3 - Soft Decision
[data3, c3, signal3, TH_signal3] = TX_BPPM_TH(nBits, Ns(2), ...
    smp_freq, Tc, Ts, Np, Nh, dPPM, IR_d, tau, powdBm);
[sRX3, a3] = pathloss(signal3, 10^(-30/20), 10, 2);
num_bits3 = length(data3);
mask3 = corrmask(TH_signal3,smp_freq, num_bits3*Ns(2), dPPM);
[sRXn3, noise3] = gnoise(sRX3, ExN0_dB, num_bits3*Ns(2));
[detected_bits3, BER3] = mod_receiver(sRXn3,mask3,smp_freq,data3,Ns(2),Ts,mod,2);
ExN0=10.^(ExN0_dB/10);
BER_theory_soft_Ns_3=0.5*erfc(sqrt(0.5*Ns(2)*ExN0));
%% Graph
if(~constant_ExN0)
    SNR_x=EbN0_dB;
    xlabelStr='E_b/N_0 [dB]';
else
    SNR_x=ExN0_dB;
    xlabelStr='E_x/N_0 [dB]';
end

figure(1)
semilogy(SNR_x, BER1,'-b','LineWidth',2); grid on; hold all;
semilogy(SNR_x, BER2, '-r','LineWidth',2); 
semilogy(SNR_x, BER3, '-m','LineWidth',2);
semilogy(SNR_x, BER_theory_Ns_1, '--ob','LineWidth',2);
semilogy(SNR_x, BER_theory_hard_Ns_3, '--or','LineWidth',2);
semilogy(SNR_x, BER_theory_soft_Ns_3, '--om','LineWidth',2);

xlabel(xlabelStr)
ylabel('BER')
set(gca,'FontSize',12)
lh=legend('PPM N_s=1','PPM N_s=3 - Hard Decision', 'PPM N_s=3 - Soft Decision', 'PPM N_s=1, Theory', 'PPM N_s=3, HD, Theory','PPM N_s=3, SD, Theory');
set(lh,'FontSize',12)
set(lh,'Location','SouthWest')