% -- BPPM_TH_mod.m
function [data, c, PPM_TH_signal, TH_signal] = TX_BPPM_TH(...
                    nBits, Ns, ...
                    fs, Tc, Tf, Np, Nh, ...
                    epsilon, Tm, tau, powdBm)
% INPUT:    
%       nBits       bitstream length                        []  
%       Ns          channel coder rep factor                [] 
%       fs          sampling frequency                      [Hz]
%       Tc          chip time                               [s]
%       Tf          frame size                              [s]
%       Np          code length                             []
%       Nh          code max value                          []
%       epsilon     PPM dither                              [s]
%       Tm          time-duration of gaussian IR            [s]
%       tau         gaussian shape par                      [s]
%       powdBm      avg power to be tx                      [dBm]
%
% OUTPUT:   data, TH code, tx signal, TH sequence

if (Nh-1)*Tc+epsilon > Tf
    warning('TX_BPPM_TH:frame_overlap','TH-UWB possible overlap between frames!');
end

data = binary_source(nBits);
rep_data = channel_coder(Ns, data);
%length(rep_data)
c = TH_code_generator(Nh, Np);
%length(c)
[PPM_TH_seq, TH_seq] = BPPM_TH_mod(rep_data, fs, Tc, Tf, epsilon, c);
p = gaussian_wf(fs, Tm, tau, 2);
PPM_TH_signal = conv(PPM_TH_seq, p);
PPM_TH_signal = PPM_TH_signal * sqrt(10^(powdBm/10) * 1e-3)*sqrt(Tf);
%PPM_TH_energy= sum(PPM_TH_signal.^2)
TH_signal = conv(TH_seq, p);
TH_signal = TH_signal * sqrt(10^(powdBm/10) * 1e-3)*sqrt(Tf);
%TH_energy= sum(TH_signal.^2)

b_plot=1;
if b_plot
% Time axis
dt = 1/fs;
Tf_idx = ceil(Tf/dt);
nSmp = length(rep_data) * Tf_idx;
t = 0:dt:nSmp*dt-dt;

% Graphics - commented out
% figure()
% stem1=stem(t*1e9, PPM_TH_seq); grid;
% title('Coded signal');
% hold on;
% stem2=stem(t*1e9, TH_seq);
% xlabel('Time (nanoseconds)');
% legend('TH+PPM','TH only');
% hold off;
% set(stem1,'LineWidth',3);
% set(stem2,'LineStyle','--','Color',[1 0.25 0.5]);
% 
% figure()
% plot(t*1e9, -PPM_TH_signal(1:length(t))); grid;
% title('TX signal');
% xlabel('Time (nanoseconds)');
% hold on;
% plot(t*1e9, -TH_signal(1:length(t)),...
%     'LineStyle','--','Color',[1 0.25 0.5]);
% legend('TH+PPM','TH only');
% hold off;
end
