% -- mod_receiver.m
function [RXbits,BER]=mod_receiver(sRx,mask,smp_freq,bits,Ns,Ts,...
                                    mod,DDT)

if mod==1 % then PPM
    sgn = +1;
end

if mod==2 % then PAM
    sgn = -1;
end

if mod~=1 && mod~=2
    warning('Wrong modulation requested! \n');
end

% DDT (Decision Detection Type): 1 (Hard), 2 (Soft)
[N,L] = size(sRx);
numbit=length(bits);
RXbits = zeros(N,numbit);
dt = 1 / smp_freq;                        % sampling time
framesamples = floor(Ts ./ dt);     % number of samples per frame
bitsamples = framesamples * Ns;     % number of samples per bit

for n = 1 : N %for each SNR value
    
    rx = sRx(n,:);
    mx = rx .* mask; %rx signal with mask
    
    if DDT == 1 % Hard Decision Detection
        
        for nb = 1 : numbit
            mxk = mx(1+(nb-1)*bitsamples:bitsamples+(nb-1)*bitsamples);
            No0 = 0;
            No1 = 0;
            
            for np = 1 : Ns
                mxkp = mxk(1+(np-1)*framesamples:...
                   framesamples+(np-1)*framesamples);
                zp = sum(mxkp.*dt);
                if sgn*zp > 0 
                    No0 = No0 + 1; 
                else      
                    No1 = No1 + 1;
                end
            end % for np = 1 : Ns
            
            if No0 > No1
                % the estimated bit is '0'
                RXbits(n,nb) = 0;
            else
                % the estimated bit is '0'
                RXbits(n,nb) = 1;
            end
            
        end % for nb = 1 : numbit
        
    end % end of Hard Decision Detection
    
    if DDT == 2 % Soft Decision Detection
        
        for nb = 1 : numbit
            
            mxk = mx(1+(nb-1)*bitsamples:bitsamples+(nb-1)*bitsamples);
            zb = sum(mxk.*dt);
            
            if sgn*zb > 0
                % the estimated bit is '0'
                RXbits(n,nb) = 0;
            else
                % the estimated bit is '1'
                RXbits(n,nb) = 1;
            end
            
        end % for nb = 1 : numbit
                
    end % end of Soft Decision Detection

end % for n = 1 : N

BER = zeros(1,N);
for n = 1 : N
    WB = sum(abs(bits-RXbits(n,:)));
    BER(n) = WB / numbit; % average Bit Error Rate
end
